#!/usr/bin/env node

//- Imports
const fs = require('fs');
const path = require('path');
const inquirer = require('inquirer');
const shell = require('shelljs');
const chalk = require('chalk');
const render = require('./utils/templates').render;

//- Obtener las opciones de los templates
const OPCIONES_PLANTILLA = fs.readdirSync(path.join(__dirname, 'template'));



//- Preguntas
const PREGUNTAS = [
	{
		name: "template",
		type: "list",
		message: "¿Qué tipo de proyecto trabajaremos en esta ocasión mi querido Genius?",
		choices: OPCIONES_PLANTILLA
	},
	{
		name: "proyecto",
		type: "input",
		message: "Bien, primero definamos el nombre de la carpeta (dejar vacio si quieres instalar en la raíz de tu carpeta)",
		validate: function(input){
			if(input.length === 0){
				return true;
			}else{
				if(/([ a-z@]{1}[a-z\-\.\\\/0-9]{0,213})+$/.test(input)){
					return true;
				}
			}
			
			return "El nombre del proyecto solo puede tener 214 caracteres, tiene que empezar en minusculas (sin numeros), no debe llevar caracteres especiales y no debe llevar espacios"
		}
	}
]

//- Pintar preguntas en consola
const DIRECTORIO_ACTUAL = process.cwd();

inquirer.prompt(PREGUNTAS).then(respuestas => {
	const plantilla = respuestas['template'];
	const proyecto = respuestas['proyecto'];
	
	const templatePath = path.join(__dirname, 'template', plantilla);
	let pathTarget = path.join(DIRECTORIO_ACTUAL, proyecto);
	
	if(proyecto.length !== 0){
		crearProyecto(pathTarget);
	}
	
	crearEstructuraDelProyecto(templatePath, proyecto);
	despuesDelProceso(templatePath, pathTarget)
});

function crearProyecto(directorioProyecto){
	// Comporbar que no existe el directorio
	if(fs.existsSync(directorioProyecto)){
		console.log(chalk.red("No puedes crear el proyecto por que ya existe, intenta con otro nombre"))
		return false;
	}
	
	fs.mkdirSync(directorioProyecto);
	return true;
}

function crearEstructuraDelProyecto(templatePath, nombreProyecto){
	const listaArchivosDirectorios = fs.readdirSync(templatePath);
	
	listaArchivosDirectorios.forEach(item => {
		const originalPath = path.join(templatePath, item);
		const stats = fs.statSync(originalPath);
		const writePath = path.join(DIRECTORIO_ACTUAL, nombreProyecto, item);
		
		if(stats.isFile()){
			let contents = fs.readFileSync(originalPath, 'utf-8');
			//contents = render(contents, {nombreProyecto});
			fs.writeFileSync(writePath, contents, 'utf-8');
			
			//Informacion Adicional
			const CREATE = chalk.green("CREANDO ");
			const size = stats['size'];
			console.log(`${CREATE} ${originalPath} (${size} bytes)`);
		} else if(stats.isDirectory()){
			fs.mkdirSync(writePath);
			crearEstructuraDelProyecto(path.join(templatePath, item), path.join(nombreProyecto, item));
		}
	});
}

function despuesDelProceso(templatePath, targetPath){
	const isNode = fs.existsSync(path.join(templatePath, "package.json"));
	
	if(isNode){
		shell.cd(targetPath);
		console.log(chalk.green(`Instalando las dependencias del proyecto en: ${targetPath}`));
		const resultado = shell.exec(`npm install`);
		if(resultado.code !== 0){
			return false;
		}
	}
}
