const path = require("path");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const webpack = require("webpack");

module.exports = {
	mode: "production",

	entry: {
		"home": path.resolve(__dirname, "src", "js", "main.js"),
		"toolkit": path.resolve(__dirname, "src", "js", "paginas", "toolkit.js"),
	},

	output: {
		path: path.resolve(__dirname, "dist/prod"),
		filename: "js/[name].js",
		publicPath: ""
	},

	devServer: {
		contentBase: path.join(__dirname, 'dist/prod'),
		port: 8000,
		writeToDisk: true
	},

	plugins: [
		new HtmlWebpackPlugin({
			template: path.resolve(__dirname, "src", "html", "index.pug"),
			filename: "index.html",
			chunks: ["home"]
		}),
		
		new HtmlWebpackPlugin({
			template: path.resolve(__dirname, "src", "html", "toolkit.pug"),
			filename: "toolkit.html",
			chunks: ["toolkit"]
		}),
		
		new MiniCssExtractPlugin({
			filename: "css/[name].css"
		}),
		
		new webpack.ProvidePlugin({
			$: 'jquery',
			jQuery: 'jquery',
		})
	],
	
	optimization: {
		splitChunks: {
			chunks: "all",
			minSize: 0,
			name: "commons"
		}
	},
	
	module: {
		rules: [
			{
				test: /\.pug$/,
				loader: "pug-loader",
				options: {
					pretty: true
				}
			},
			
			{
				test: /\.(styl)$/,
				use: [
					{
						loader: MiniCssExtractPlugin.loader
					},
					{
						loader: "css-loader",
						options: {
							url: false
						}
					},
					{
						loader: "stylus-loader"
					}
				]
			},
			
			{
				test: /\.(less)$/,
				use: [
					{
						loader: MiniCssExtractPlugin.loader
					},
					{
						loader: "css-loader"
					},
					{
						loader: "less-loader"
					}
				]
			},
			
			{
				test: /\.(scss)$/,
				use: [
					{
						loader: MiniCssExtractPlugin.loader
					},
					{
						loader: "css-loader"
					},
					{
						loader: "sass-loader"
					}
				]
			},
			
			{
				test:  /\.js$/,
				exclude: /node_modules/,
				use: [
					{
						loader: "babel-loader"
					}
				]
			},
			
			{
				test:  /\.(jpg?g|png|svg|gif|mp4|webm|eot|ttf|woff)$/,
				include: path.join(__dirname, 'src'),
				use: [
					{
						loader: "url-loader",
						options: {
							encoding: false,
							limit: false,
							name: "[path][name].[ext]]",
							context: "src"
						}
					},
					{
						loader: "file-loader",
						options: {
							name: "[path][name].[ext]",
							context: "src"
						}
					}
				]
			}
		]
	}
}
